using System;

namespace StatePatternDemo
{
    public abstract class FlightState
    {
        protected FlightState(int flightNumber)
        {
            FlightNumber = flightNumber;
        }

        public abstract bool IsOnTime();
        public abstract bool IsInTransit();

        public Int32 FlightNumber { get; private set; }

        private DateTime _arrivalTime = DateTime.MaxValue;
        public virtual DateTime ArrivalTime
        {
            get { return _arrivalTime; }
            set { _arrivalTime = value; }
        }

        private DateTime _departureTime = DateTime.MinValue;
        public virtual DateTime DepartureTime
        {
            get { return _departureTime; }
            set { _departureTime = value; }
        }

        private DelayReasons _delayReason = DelayReasons.None;
        public DelayReasons DelayReason
        {
            get { return _delayReason; }
            internal set { _delayReason = value; }
        }
    }

    public class WeatherDelayState : FlightState
    {
        public WeatherDelayState(int flightNumber, DelayReasons delayReasons) : base(flightNumber)
        {
            DelayReason = delayReasons;
        }

        public override bool IsOnTime()
        {
            return false;
        }

        public override bool IsInTransit()
        {
            return true;
        }
    }

    public class IdleState : FlightState
    {
        public IdleState(int flightNumber) : base(flightNumber)
        {
        }

        public override bool IsOnTime()
        {
            return true;
        }

        public override bool IsInTransit()
        {
            return false;
        }
    }

    public class InTransitState : FlightState
    {
        public InTransitState(int flightNumber) : base(flightNumber)
        {
        }

        public override bool IsOnTime()
        {
            return true;
        }

        public override bool IsInTransit()
        {
            return true;
        }
    }
}